#!/usr/bin/python
# vi:si:et:sw=4:sts=4:ts=4
# -*- coding: utf-8 -*-

from __future__ import division
import os
import sys
import struct 

import gobject
gobject.threads_init()

import pygst
pygst.require('0.10')
import gst
from gst.extend import discoverer


codec_list = {
    'MPEG-1 layer 3': 'mp3',
    'MPEG-1 layer 3 audio': 'mp3',
    'MPEG 2 Audio, Layer 3 (MP3)': 'mp3',
    'MPEG 1 Audio, Layer 2': 'mp2',
    'VP6 Flash video': 'vp6',
    'On2 VP6 Video': 'vp6',
    'AC-3 audio': 'ac3',
    'MPEG-4 AAC audio': 'aac',
    'H.264 / AVC': 'h264',
    'Raw 16-bit PCM audio': 'pcm_s16le',
    'Generic DV': 'dvvideo',
    'Motion-JPEG': 'mjpeg',
}

class AVInfo(dict):
    def __init__(self, path):
        self['size'] = os.stat(path).st_size
        mainloop = gobject.MainLoop()
        def discovered(d, is_media):
            if is_media:
                if d.is_video:
                    self['width'] = d.videowidth
                    self['height'] = d.videoheight
                    self['duration'] = float(d.videolength)/gst.SECOND
                    self["framerate"] = "%d:%d" % (d.videorate.num, d.videorate.denom)
                    s = d.videocaps[0]
                    par = gst.Fraction(1, 1)
                    if 'pixel-aspect-ratio' in s.keys():
                        par = s['pixel-aspect-ratio']
                    self["pixel_aspect_ratio"] = "%s:%s" %(par.num, par.denom)
                    dar = gst.Fraction(d.videowidth * par.num, d.videoheight * par.denom)
                    if 'display-aspect-ratio' in s.keys():
                        dar = s['display-aspect-ratio']
                    self["display_aspect_ratio"] = "%s:%s" %(dar.num, dar.denom)
                if d.is_audio:
                    #self['audio caps'] = d.audiocaps
                    #self['audio format'] = d.audiofloat and 'floating-point' or 'integer'
                    #self['sample width (bits)'] = d.audiowidth
                    #self['sample depth (bits)'] = d.audiodepth
                    self['samplerate'] = d.audiorate
                    if 'duration' not in self:
                        self['duration'] = float(d.audiolength)/gst.SECOND
                    self['channels'] = d.audiochannels

                if 'video-codec' in d.tags:
                    self['video_codec'] = codec_list.get(d.tags['video-codec'], d.tags['video-codec'])
                if 'audio-codec' in d.tags:
                    self['audio_codec'] = codec_list.get(d.tags['audio-codec'], d.tags['audio-codec'])
            mainloop.quit()

        d = discoverer.Discoverer(path)
        d.connect('discovered', discovered)
        d.discover()
        mainloop.run()
        self['oshash'] = self._oshash(path)
        self['path'] = path

    def _oshash(self, name): 
        try: 
            longlongformat = 'q'  # long long 
            bytesize = struct.calcsize(longlongformat) 
                
            f = open(name, "rb") 
                
            filesize = os.path.getsize(name) 
            hash = filesize 
                
            if filesize < 65536 * 2: 
               return "SizeError" 
             
            for x in range(int(65536/bytesize)): 
                buffer = f.read(bytesize) 
                (l_value,)= struct.unpack(longlongformat, buffer)  
                hash += l_value 
                hash = hash & 0xFFFFFFFFFFFFFFFF #to remain as 64bit number  
                     

            f.seek(max(0,filesize-65536),0) 
            for x in range(int(65536/bytesize)): 
                buffer = f.read(bytesize) 
                (l_value,)= struct.unpack(longlongformat, buffer)  
                hash += l_value 
                hash = hash & 0xFFFFFFFFFFFFFFFF 
             
            f.close() 
            returnedhash =  "%016x" % hash 
            return returnedhash 
        except(IOError): 
            return "IOError"


